/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2007, 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
var TreeChooserUtil = new Object();

////////////////////////////////////////////////////////////////////////////////
// Criteria Chooser functions
////////////////////////////////////////////////////////////////////////////////

TreeChooserUtil.markNextSibling = function(tree, currentNode){
    var foundNext = false;
    var sibling = currentNode.nextSibling;
    // find a sibling after this to mark
    while(sibling != null && !foundNext){
        if(sibling.nodeName == 'LI'){
        if(tree_isAvailable(sibling)){
                foundNext = true;
            }
            else if(tree_hasSelectedChildren(sibling)){
                foundNext = true;
            }
        }
        if(!foundNext){
            sibling = sibling.nextSibling;
        }
    }
    if(!foundNext){
        sibling = currentNode.previousSibling;
    }
    // find a sibling before this to mark
    while(sibling != null && !foundNext){
        if(sibling.nodeName == 'LI'){
        	if(tree_isAvailable(sibling)){
                foundNext = true;
            }
            else if(tree_hasSelectedChildren(sibling)){
                foundNext = true;
            }
        }
        if(!foundNext){
            sibling = sibling.previousSibling;
        }
    }
    // mark the sibling
    if(foundNext){
    	treeTable_markNode(tree, sibling);
    }
    return !foundNext;
}

TreeChooserUtil.removeNode = function(tree, node){
	treeTable_unmarkNode(tree, node);
	tree_removeNode(node);
	return TreeChooserUtil.cleanParent(tree, node);
}

TreeChooserUtil.cleanParent = function(tree, node){
	var parentNode = tree_getParent(node);
	if(!parentNode || tree_hasSelectedChildren(parentNode)){
		// no change, return the child
		return node; 
	}
	else {
		// clean empty folder
		return TreeChooserUtil.removeNode(tree, parentNode);
	}
}

TreeChooserUtil.createOptionsFromSelectedNodes = function(tree){
	var nodes = new Array();
	var items = tree.selectedNode;
    for(var i = 0; i < items.length; i++){
        var nodeid = items[i];
	   	var node = document.getElementById(nodeid);
	   	nodes[nodes.length] = node;
    }
	return TreeChooserUtil.createOptionsFromNodes(nodes);
}

TreeChooserUtil.createOptionsFromNodes = function(nodes){
	var options = new Array();
    for(var i = 0; i < nodes.length; i++){
	   	var node = nodes[i];
        var id = node_nodeId(node);
	    // only add the currently selected node to the array
	    var mylabel = tree_getNodeLabel(node);
	    var opt = new Option(mylabel, id);
	    options[options.length] = opt;
    }
	return options;
}

// Dual List Chooser Interface 
//overriding functions from the USMiDualListChooser object
function USMiTreeChooser(isDualTree, dualListChooserVar, nodeBehavior, availableTreeId, selectedTreeId) {
  this.dualListChooserVar = dualListChooserVar;

  if (dualListChooserVar != null) {
    if (isDualTree) {
      dualListChooserVar.addToSelList = this.addToSelList;
      dualListChooserVar.removeFromSelList = this.removeFromSelList;
      dualListChooserVar.buttonStates = this.buttonStates;
      if (dualListChooserVar.addType == 'move') {
        dualListChooserVar.populateCurrSelSelItems = function() {
          //no op
        };
      }
    }
    else {
    	dualListChooserVar.ADD_HANDLER = this.ADD_HANDLER;
    }

    dualListChooserVar.addButtonState = this.addButtonState;
    dualListChooserVar.REMOVE_FROM_AVAIL = this.REMOVE_FROM_AVAIL;
    dualListChooserVar.ADD_TO_AVAIL = this.ADD_TO_AVAIL;    
    
    //adding new properties to the USMiDualListChooser object
    dualListChooserVar.availableTreeId = availableTreeId;
    dualListChooserVar.selectedTreeId = selectedTreeId;
    dualListChooserVar.isDualTree = isDualTree;
    // node behavior function
    if(nodeBehavior == 'node'){
      dualListChooserVar.behavior = this.behaviorNode;
      dualListChooserVar.addBehavior = this.addBehaviorNode;
      dualListChooserVar.removeBehavior = this.removeBehaviorNode;
    }
    else if (nodeBehavior == 'children') {
      dualListChooserVar.behavior = this.behaviorChildren;
      dualListChooserVar.addBehavior = this.addBehaviorChildren;
      dualListChooserVar.removeBehavior = this.removeBehaviorChildren;
	} else { // all
      dualListChooserVar.behavior = this.behaviorAll;
      dualListChooserVar.addBehavior = this.addBehaviorAll;
      dualListChooserVar.removeBehavior = this.removeBehaviorAll;
	}
  }
}

USMiTreeChooser.prototype.addToSelList = function() {
  this.CURR_SEL_SEL_ITEMS = this.addBehavior(this.selectedTreeId, this.CURR_SEL_AVAIL_ITEMS);
  // this.CURR_SEL_SEL_ITEMS = TreeChooserUtil.addNodes(this.selectedTreeId, nodes);
  this.REMOVE_FROM_AVAIL();
}

// Remove user selected items from the selected list, and add to the available list (move)
USMiTreeChooser.prototype.removeFromSelList = function() {
  this.ADD_TO_AVAIL();
  this.CURR_SEL_SEL_ITEMS = this.removeBehavior(this.selectedTreeId, this.CURR_SEL_AVAIL_ITEMS);
  //this.CURR_SEL_SEL_ITEMS = TreeChooserUtil.removeNodes(this.selectedTreeId, nodes);
  this.buttonStates();
}

// Handles button enabling/disabling - adapted from WCL source
USMiTreeChooser.prototype.buttonStates = function() {
  this.addButtonState();
  var remButton = document.getElementById(this.removeButtonId);
  if (remButton != null) {
    remButton.disabled = this.CURR_SEL_SEL_ITEMS.length == 0;
  }
}

USMiTreeChooser.prototype.addButtonState = function() {
  var addButton = document.getElementById(this.addButtonId);
  if (addButton != null) {

    var addDisabled = null;

	// TODO check on tree chooser when addType is copy and tree item is selected
    if (this.isDualTree || this.addType == "move") {
      var options = this.CURR_SEL_AVAIL_ITEMS; 
      addDisabled = options.length == 0;
      for(var i=0; i<options.length; i++) {
        var nodeid = this.availableTreeId + "_" + options[i].value;
        var node = document.getElementById(nodeid);
        if(!node_isLeafNode(node)){
          var children = node.getElementsByTagName('LI');
          if (children.length === 0) {
            addDisabled = true;
            break;
          }
        }
      }
    } else {
      addDisabled = false;
      var selList = document.getElementById(this.SEL_OBJ);
      for (var i = 0; i < this.CURR_SEL_AVAIL_ITEMS.length; i++) {
        var nodeId = this.CURR_SEL_AVAIL_ITEMS[i].value;
        for (var j = 0; j < selList.length; j++) {
          if(nodeId == selList.options[j].value) {
            addDisabled = true;
          }
        }
      }
    }

    addButton.disabled = addDisabled;
  }
}

USMiTreeChooser.prototype.onclickSelectedDualTree = function(e) {
  this.onclick(e);
  // iterate through the selected nodes and build the selected list
  var tree = document.getElementById(this.dualListChooserVar.selectedTreeId);
  
  this.dualListChooserVar.CURR_SEL_SEL_ITEMS = TreeChooserUtil.createOptionsFromSelectedNodes(tree);
  this.dualListChooserVar.buttonStates();
}

USMiTreeChooser.prototype.onkeypressSelectedDualTree = function(e) {
  this.onkeypress(e);
  // iterate through the selected nodes and build the selected list
  var tree = document.getElementById(this.dualListChooserVar.selectedTreeId);
  this.dualListChooserVar.CURR_SEL_SEL_ITEMS = TreeChooserUtil.createOptionsFromSelectedNodes(tree);
  this.dualListChooserVar.buttonStates();
}

USMiTreeChooser.prototype.onclickAvailableDualTree = function(e) {
  this.onclick(e);
  // iterate through the selected nodes and build the selected list
  var tree = document.getElementById(this.dualListChooserVar.availableTreeId);
  this.dualListChooserVar.CURR_SEL_AVAIL_ITEMS = TreeChooserUtil.createOptionsFromSelectedNodes(tree);
  this.dualListChooserVar.buttonStates();
}

USMiTreeChooser.prototype.onkeypressAvailableDualTree = function(e) {
  this.onkeypress(e);
  // iterate through the selected nodes and build the selected list
  var tree = document.getElementById(this.dualListChooserVar.availableTreeId);
  this.dualListChooserVar.CURR_SEL_AVAIL_ITEMS = TreeChooserUtil.createOptionsFromSelectedNodes(tree);
  this.dualListChooserVar.buttonStates();
}

USMiTreeChooser.prototype.onclickAvailableTreeList = function(e) {
  this.onclick(e);
  // iterate through the selected nodes and build the selected list
  var tree = document.getElementById(this.dualListChooserVar.availableTreeId);
  this.dualListChooserVar.CURR_SEL_AVAIL_ITEMS = TreeChooserUtil.createOptionsFromSelectedNodes(tree);
  this.dualListChooserVar.buttonStates();
}

USMiTreeChooser.prototype.onkeypressAvailableTreeList = function(e) {
  this.onkeypress(e);
  // iterate through the selected nodes and build the selected list
  var tree = document.getElementById(this.dualListChooserVar.availableTreeId);
  this.dualListChooserVar.CURR_SEL_AVAIL_ITEMS = TreeChooserUtil.createOptionsFromSelectedNodes(tree);
  this.dualListChooserVar.buttonStates();
}

USMiTreeChooser.prototype.onclick = function(e) {
  var node = tree_findNodeForSelectEvent(e);
  var table = document.getElementById(node.treeId);
  // call the tree api
  treeTable_clickNode(e, table.id, node.id);
}

USMiTreeChooser.prototype.onkeypress = function(e) {
  var node = tree_findNodeForSelectEvent(e);
  // call the tree api
  var table = document.getElementById(node.treeId);
  var keynum;
  var keychar;
  var numcheck;
  if(window.event) {
    keynum = e.keyCode;
  }
  else if(e.which){
    keynum = e.which;
  }
  keychar = String.fromCharCode(keynum);
  table.clickType = 'click';
  table.ctrlClick = e.ctrlKey;
  if(keychar == ' ') {
	treeTable_markNode(table, node);
  }
}

USMiTreeChooser.prototype.ADD_TO_AVAIL = function() {
  this.CURR_SEL_AVAIL_ITEMS = this.addBehavior(this.availableTreeId, this.CURR_SEL_SEL_ITEMS);
  if(!this.isDualTree){
  	this.CURR_SEL_SEL_ITEMS = new Array();
  }
}

USMiTreeChooser.prototype.REMOVE_FROM_AVAIL = function() {
  if (this.addType == 'move') {
  	if(!this.isDualTree){
    	this.CURR_SEL_AVAIL_ITEMS = this.removeBehavior(this.availableTreeId, this.LIST_CACHE_SEL_AVAIL_ITEMS);
    	this.LIST_CACHE_SEL_AVAIL_ITEMS = null;
  	}
  	else {
    	this.CURR_SEL_AVAIL_ITEMS = this.removeBehavior(this.availableTreeId, this.CURR_SEL_SEL_ITEMS);
  	}
    this.buttonStates();
  }
}

USMiTreeChooser.prototype.addBehaviorAll = function(toTreeId, options) {
	var tree = document.getElementById(toTreeId);
	treeTable_unmarkAll(tree);
	// fake the ctlr click
    tree.ctrlClick = true;
    tree.clickType = 'click';
	var nodes = new Array();
	for(var i=0; i<options.length; i++) {
		var nodeid = toTreeId + "_" + options[i].value;
		var node = document.getElementById(nodeid);
		nodes[nodes.length] = node;
		if(!node_isLeafNode(node)){
			var children = node.getElementsByTagName('LI');
            for(var j = 0; j < children.length; j++){
				var child = children[j];
				if(j < 1){
					tree_expandAllParents(tree, child);
				}
				tree_addNode(child, true);
			}
		}
		tree_expandAllParents(tree, node);
		tree_addNode(node, true);
		treeTable_markNode(tree, node);
	}
	return TreeChooserUtil.createOptionsFromSelectedNodes(tree);
}

USMiTreeChooser.prototype.addBehaviorChildren = function(toTreeId, options) {
	var tree = document.getElementById(toTreeId);
	treeTable_unmarkAll(tree);
	// fake the ctlr click
    tree.ctrlClick = true;
    tree.clickType = 'click';
	var nodes = new Array();
	for(var i=0; i<options.length; i++) {
		var nodeid = toTreeId + "_" + options[i].value;
		var node = document.getElementById(nodeid);
		tree_expandAllParents(tree, node);
		treeTable_markNode(tree, node);
		if(node_isLeafNode(node)){
			tree_addNode(node, true);
		}
		else {
			var children = node.getElementsByTagName('LI');
			for(var j = 0; j < children.length; j++){
				var child = children[j];
				tree_expandAllParents(tree, child);
				tree_addNode(child, true);
			}
		}
	}
	return TreeChooserUtil.createOptionsFromSelectedNodes(tree);
}

USMiTreeChooser.prototype.addBehaviorNode = function(toTreeId, options) {
	var tree = document.getElementById(toTreeId);
	treeTable_unmarkAll(tree);
	// fake the ctlr click
    tree.ctrlClick = true;
    tree.clickType = 'click';
	var nodes = new Array();
	for(var i=0; i<options.length; i++) {
		var nodeid = toTreeId + "_" + options[i].value;
		var node = document.getElementById(nodeid);
		tree_expandAllParents(tree, node);
		tree_addNode(node, true);
		treeTable_markNode(tree, node);
	}
	return TreeChooserUtil.createOptionsFromSelectedNodes(tree);
}

USMiTreeChooser.prototype.removeBehaviorAll = function(fromTreeId, options) {
	var tree = document.getElementById(fromTreeId);
	var lastNode = null;
	var nodes = new Array();
	for(var i=0; i<options.length; i++) {
		var nodeid = fromTreeId + "_" + options[i].value;
		var node = document.getElementById(nodeid);
		if(!node_isLeafNode(node)){
			var children = node.getElementsByTagName('LI');
			for(var j = 0; j < children.length; j++){
				lastNode = TreeChooserUtil.removeNode(tree, children[j]);
			}
		}
		lastNode = TreeChooserUtil.removeNode(tree, node);
	}
	treeTable_unmarkAll(tree);
	// If we removed any nodes, mark the sibling of the last node
	if(lastNode){
		TreeChooserUtil.markNextSibling(tree, lastNode);
	}
	return TreeChooserUtil.createOptionsFromSelectedNodes(tree);
}

USMiTreeChooser.prototype.removeBehaviorChildren = function(fromTreeId, options) {
	var tree = document.getElementById(fromTreeId);
	var lastNode = null;
	var nodes = new Array();
	for(var i=0; i<options.length; i++) {
		var nodeid = fromTreeId + "_" + options[i].value;
		var node = document.getElementById(nodeid);
		if(node_isLeafNode(node)){
			lastNode = TreeChooserUtil.removeNode(tree, node);
		}
		else {
			var children = node.getElementsByTagName('LI');
			for(var j = 0; j < children.length; j++){
				lastNode = TreeChooserUtil.removeNode(tree, children[j]);
			}
		}
	}
	treeTable_unmarkAll(tree);
	// If we removed any nodes, mark the sibling of the last node
	if(lastNode){
		TreeChooserUtil.markNextSibling(tree, lastNode);
	}
	return TreeChooserUtil.createOptionsFromSelectedNodes(tree);
}

USMiTreeChooser.prototype.removeBehaviorNode = function(fromTreeId, options) {
	alert("Remove on node behavior is not supported.");
	return options;
}

USMiTreeChooser.prototype.ADD_HANDLER = function(thisEvent) {
	var nodes = this.behavior(this.availableTreeId, this.CURR_SEL_AVAIL_ITEMS);
	this.CURR_SEL_AVAIL_ITEMS = TreeChooserUtil.createOptionsFromNodes(nodes);
	this.LIST_CACHE_SEL_AVAIL_ITEMS = this.CURR_SEL_AVAIL_ITEMS.slice();
}

USMiTreeChooser.prototype.behaviorAll = function(toTreeId, options) {
	var nodes = new Array();
	for(var i=0; i<options.length; i++) {
		var nodeid = toTreeId + "_" + options[i].value;
		var node = document.getElementById(nodeid);
		nodes[nodes.length] = node;
		if(!node_isLeafNode(node)){
			var children = node.getElementsByTagName('LI');
			for(var j = 0; j < children.length; j++){
				nodes[nodes.length] = children[j];
			}
		}
	}
	return nodes;
}

USMiTreeChooser.prototype.behaviorChildren = function(toTreeId, options) {
	var nodes = new Array();
	for(var i=0; i<options.length; i++) {
		var nodeid = toTreeId + "_" + options[i].value;
		var node = document.getElementById(nodeid);
		if(node_isLeafNode(node)){
			nodes[nodes.length] = node;
		}
		else {
			var children = node.getElementsByTagName('LI');
			for(var j = 0; j < children.length; j++){
				nodes[nodes.length] = children[j];
			}
		}
	}
	return nodes;
}

USMiTreeChooser.prototype.behaviorNode = function(toTreeId, options) {
	var nodes = new Array();
	for(var i=0; i<options.length; i++) {
		var nodeid = toTreeId + "_" + options[i].value;
		var node = document.getElementById(nodeid);
		nodes[nodes.length] = node;
	}
	return nodes;
}
