/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2008, 2011 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
function USMiDualListChooser(clientId, addType, sortType) {

  this.clientId = clientId;
  this.SEL_OBJ = clientId + '_selObj';
  this.AVAIL_OBJ = clientId + '_availObj';
  this.addButtonId = clientId + '_add';
  this.removeButtonId = clientId + '_remove';
  this.upButtonId = clientId + '_up';
  this.downButtonId = clientId + '_down';
  this.AVAIL_ITEMS = clientId + '_availItems';
  this.SEL_ITEMS = clientId + '_selItems';

  //hooks functions for component that wish to extend DualListChooser behavior
  this.ADD_HANDLER = null;
  this.REMOVE_HANDLER = null;

  this.ADD_TO_AVAIL = null;
  this.REMOVE_FROM_AVAIL = null;
  this.ADD_TO_SELECTED = null;
  this.REMOVE_FROM_SELECTED = null;

  this.CURR_SEL_AVAIL_ITEMS = new Array();
  this.CURR_SEL_SEL_ITEMS = new Array();

  this.addType = addType.toLowerCase();
  this.sortType = sortType.toLowerCase();
}

/**
 * Ali: Add title to all items
 */
USMiDualListChooser.prototype.addTitle = function() {

  var selectedList = document.getElementById(this.SEL_OBJ);
  var availableList = document.getElementById(this.AVAIL_OBJ);

  if (availableList != null ) {
    for ( var i = 0; i < availableList.length; i++ ) {
      availableList.options[i].title = availableList.options[i].text;
    }
  }

  if (selectedList != null ) {
    for ( var i = 0; i < selectedList.length; i++ ) {
      selectedList.options[i].title = selectedList.options[i].text;
    }
  }
}

// Remove selected items from the Selected Items list
USMiDualListChooser.prototype.removeFromSelList = function(thisEvent) {

  if (this.REMOVE_HANDLER != null) {
    this.REMOVE_HANDLER(thisEvent);
  }

  var selList = document.getElementById(this.SEL_OBJ);
  var lastIndex = 0;
  for(var i = selList.length - 1; i >= 0; i--) {
    if(selList.options[i].selected) {
      selList.options[i].selected = false;
      lastIndex = i;
      if(selList.length > 0) {
        selList.options[i] = null;
      }
    }
  }

  if (selList.length > 0) {
    if (lastIndex >= selList.length) {
      lastIndex = selList.length - 1;
    }
  }

  this.updateSelHidden();
  if (this.ADD_TO_AVAIL != null) {
    this.ADD_TO_AVAIL();
  }
  this.addTitle();
  this.buttonStates();
}

// Handler for Up and Down buttons
// from WCL client-side dual list chooser implementation
USMiDualListChooser.prototype.reorder = function(direction) {
  var listBox = document.getElementById(this.SEL_OBJ);
  if ( listBox != null ) {
    var start = (direction < 0) ? 0 : listBox.options.length - 1;
    var end   = (direction < 0) ? listBox.options.length : -1;

    while ((start != end) && (0 <= start) && (start < listBox.options.length) && listBox.options[start].selected) {
	    start -= direction;
    }

    for (var i = start; (i != end) && (0 <= i) && (i < listBox.options.length); i -= direction ) {
      if (listBox.options[i].selected) {
        var partner = i + direction;

        if ((0 <= partner) && (listBox.options.length > partner)) {
          var temp1 = listBox.options[i];
          var tempText = temp1.text;
          var tempValue = temp1.value;
          var tempSelect = temp1.selected;
          var temp2 = listBox.options[partner];
          temp1.text = temp2.text;
          temp1.value = temp2.value;
          temp1.title = temp2.text;
          temp1.selected= temp2.selected;
          temp2.text = tempText;
          temp2.value = tempValue;
          temp2.title = tempText;
          temp2.selected = tempSelect;

        }
      }
    }
  }
  this.sortButtonStates();
  this.updateSelHidden();
}

// Updates a hidden field with the contents of the Selected Items list
USMiDualListChooser.prototype.updateSelHidden = function() {
  var selHidden = document.getElementById(this.SEL_ITEMS);
  selHidden.value = '';
  var selList = document.getElementById(this.SEL_OBJ);
  for(var i=0; i < selList.options.length; i++) {
    selHidden.value = selHidden.value + selList.options[i].value + '\t'+ selList.options[i].text + '\t';
  }
}

// Clears selections in a list
USMiDualListChooser.prototype.clearSelections = function(list) {
  for(var i=0; i < list.options.length; i++) {
    if (list.options[i].selected == true) {
      list.options[i].selected = false;
    }
  }
}

// onfocus select the first item of the list, t
// This is to fix Accessibility issue
USMiDualListChooser.prototype.onfocusAvailableItems = function() {
     this.addTitle();
  	if(hX_6.brw.isIE()) {
    	return;
  	}
    var list = document.getElementById(this.AVAIL_OBJ);
    var selectedItem = this.isItemSelected(list);
    if(selectedItem == -1 && list.size > 0 ) {
        list.selectedIndex = 0;
    }
     this.buttonStates();
     this.boxToggle(true);
}

// onfocus select the first item of the list
// This is to fix Accessibility issue
USMiDualListChooser.prototype.onfocusSelectedItems = function() {
     this.addTitle();
	if(hX_6.brw.isIE()) {
	   	return;
	}

    var list = document.getElementById(this.SEL_OBJ);
    var selectedItem = this.isItemSelected(list);
    if(selectedItem == -1 && list.size > 0 ) {
        list.selectedIndex = 0;
    }
    this.buttonStates();
    this.boxToggle(false);
}

// ia item selected, this is used in onfocusSelect
USMiDualListChooser.prototype.isItemSelected = function(list) {
  for(var i=0; i < list.options.length; i++) {
    if (list.options[i].selected == true) {
        return i;
    }
  }
  return -1;
}


USMiDualListChooser.prototype.buttonStates = function() {

  this.addButtonState();

  var rightBox = document.getElementById(this.SEL_OBJ);
  var remButton = document.getElementById(this.removeButtonId);
  if (rightBox != null && remButton != null) {
    remButton.disabled = (-1 == rightBox.selectedIndex);
  }

  if (this.sortType == 'user') {
    this.sortButtonStates();
  }
}

USMiDualListChooser.prototype.addButtonState = function() {
  var leftBox = document.getElementById(this.AVAIL_OBJ);
  var addButton = document.getElementById(this.addButtonId);
  if (leftBox != null && addButton != null) {
    addButton.disabled = (-1 == leftBox.selectedIndex);
  }
}

// Handles Up and Down button enabling/disabling
USMiDualListChooser.prototype.sortButtonStates = function() {

  var upButton = document.getElementById(this.upButtonId);
  var dnButton = document.getElementById(this.downButtonId);
  var listBox = document.getElementById(this.SEL_OBJ);
  upButton.disabled = ((-1 == listBox.selectedIndex) || (0 == listBox.selectedIndex));
  dnButton.disabled = ((-1 == listBox.selectedIndex) || ((listBox.length-1) == listBox.selectedIndex));
}

// Handles hiding the specified box if empty; displaying if not
// if (UIUtils.isIE(FacesContext.getCurrentInstance())) {
USMiDualListChooser.prototype.boxToggle = function(isAvailBox) {
  if(hX_6.brw.isIE()) {
    return;
  }

  var box = document.getElementById(this.SEL_OBJ);
  if (isAvailBox) {
    box = document.getElementById(this.AVAIL_OBJ);
  }
  if (box.options.length == 0) {
    box.style.visibility = 'hidden';
  } else {
    box.style.visibility = 'visible';
  }
}

// Searches for an Option in an array with the same value and returns
// the first index of a the found Option or -1 if not found
USMiDualListChooser.prototype.arrayContains = function(anArray, opt) {
  for (var i=0; i < anArray.length; i++) {
    if (anArray[i].value == opt.value) {
      return i;
    }
  }
  return -1;
}

// Move selected items between lists.
// Adapted from WCL client-side dual list chooser implementation.
USMiDualListChooser.prototype.moveItems = function(isLeftBox) {

  var fromName = (isLeftBox) ? this.AVAIL_OBJ : this.SEL_OBJ;
  var toName = (isLeftBox) ? this.SEL_OBJ : this.AVAIL_OBJ;
  var fromList = document.getElementById(fromName);
  var toList = document.getElementById(toName);

  if (toList != null && fromList != null) {
    toList.selectedIndex = -1;
    var lastIndex = -1;
    var index = fromList.selectedIndex;

    while (-1 != index) {
      lastIndex = index;
      var selectedOption = fromList.options[index];

      // Don't copy an item if it's already in the list
      if (this.addType == 'copy') {
        var dupIndex = this.arrayContains(toList.options, selectedOption);
        if (dupIndex == -1) {
          var newOption = new Option( selectedOption.text, selectedOption.value );
          toList.options[toList.options.length] = newOption;
          toList.options[toList.options.length - 1].selected = true;
        }
        // if copy mode, deselect the selected item to prevent infinite loop
        fromList.options[fromList.selectedIndex].selected = false;

      } else {
        var newOption = new Option( selectedOption.text, selectedOption.value );
        toList.options[toList.options.length] = newOption;
        toList.options[toList.options.length - 1].selected = true;
      }

      if (this.addType == 'move') {
        fromList.options[index] = null;
      }

      index = fromList.selectedIndex;
    }

    if ((-1 != lastIndex) && (0 < fromList.options.length)) {
      if (lastIndex >= fromList.options.length) {
        lastIndex--;
      }
      fromList.options[lastIndex].selected = true;
    }
    this.updateHiddens();
    this.addTitle();
    this.buttonStates();

    this.boxToggle(false);
    this.boxToggle(true);
  }
}

// Update both hidden fields - available and selected
USMiDualListChooser.prototype.updateHiddens = function() {
  this.updateSelHidden();
  this.updateAvailHidden();
}

// Updates a hidden field with the contents of the Available Items list
USMiDualListChooser.prototype.updateAvailHidden = function() {

  var availHidden = document.getElementById(this.AVAIL_ITEMS);
  availHidden.value = "";
  var availList = document.getElementById(this.AVAIL_OBJ);

  for(var i=0; i < availList.options.length; i++) {
    availHidden.value = availHidden.value + availList.options[i].value + '\t' + availList.options[i].text + '\t';
  }
}

// Get items from JS var CURR_SEL_AVAIL_ITEMS and add to selected listbox
USMiDualListChooser.prototype.addToSelList = function(thisEvent) {

  if (this.ADD_HANDLER != null) {
    this.ADD_HANDLER(thisEvent);
  }

  var selList = document.getElementById(this.SEL_OBJ);
  this.clearSelections(selList);

  for (var i = 0; i < this.CURR_SEL_AVAIL_ITEMS.length; i++) {
    var opt = new Option(this.CURR_SEL_AVAIL_ITEMS[i].text, this.CURR_SEL_AVAIL_ITEMS[i].value);
    var index = this.arrayContains(selList.options, opt);
    if (index == -1) {
      selList.options[selList.length] = opt;
    }
  }
  this.updateSelHidden();

  if (this.addType == 'move' && this.REMOVE_FROM_AVAIL != null) {
    this.REMOVE_FROM_AVAIL();
  }

  this.buttonStates();
  this.addTitle();
  this.boxToggle(false);
}

USMiDualListChooser.prototype.populateCurrSelSelItems = function(thisEvent) {

  if (this.CURR_SEL_SEL_ITEMS != null) {
    var selText = new Array();
    var selVals = new Array();
    var selCt = 0;
    var i = 0;
    var selList = document.getElementById(this.SEL_OBJ);

    for (i = 0; i < selList.length; i++) {
      if(selList.options[i].selected) {
        selText[selCt] = selList.options[i].text;
        selVals[selCt] = selList.options[i].value;
        selCt++;
      }
    }

    for(i = selCt - 1; i >= 0; i--) {
      var newOpt = new Option(selText[i], selVals[i]);
      this.CURR_SEL_SEL_ITEMS[i] = newOpt;
    }
  }
}

USMiDualListChooser.prototype.clearCurrSelAvailItems = function() {
  this.CURR_SEL_AVAIL_ITEMS.length = 0;
}

/**
 * Adds an option to end the of the currently selected available items array.
 */
USMiDualListChooser.prototype.addSelAvailItem = function(optionText, optionValue) {
  var newOpt = new Option(optionText, optionValue);
  this.CURR_SEL_AVAIL_ITEMS[this.CURR_SEL_AVAIL_ITEMS.length] = newOpt;
}
