/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 * 
 * (C) Copyright IBM Corp. 2010, 2011 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
/* IBM_COPYRIGHT_END */

/* http://www.kryogenix.org/code/browser/searchhi/ */
/* Modified 20021006 to fix query string parsing and add case insensitivity */
function highlightWord(node,word) {


	//do not highlight columns that are not searchable
    if(node.id != null && (node.id.lastIndexOf("_notSearchableColumn") != -1))
    {
    	return;
    }
    
    // Iterate into this nodes childNodes
    
	if (node.hasChildNodes) {
		for (var hi_cn=0;hi_cn<node.childNodes.length;hi_cn++) {
			highlightWord(node.childNodes[hi_cn],word);
		}
	}
		
	var tempWordVal = word.toLowerCase();	
	// And do this node itself
	if (node.nodeType == 3) { // text node
	
		var tempNodeVal = node.nodeValue.toLowerCase();
		if (tempNodeVal == null || tempNodeVal.length == 0){
			return;
		}
		if (tempNodeVal.indexOf(tempWordVal) != -1) {			
		
			var pn = node.parentNode;
			
			//fix for 86128 combo box problem
			if(pn.tagName.toLowerCase() == "option")
				return;
			
			if (pn.className != "searchword") {
			
				// word has not already been highlighted!
				var nv = node.nodeValue;
				var ni = tempNodeVal.indexOf(tempWordVal);
				
				// Create a load of replacement nodes
				var before = document.createTextNode(nv.substr(0,ni));
				var docWordVal = nv.substr(ni,word.length);
				var after = document.createTextNode(nv.substr(ni+word.length));
				var hiwordtext = document.createTextNode(docWordVal);
				var hiword = document.createElement("span");
				hiword.className = "searchword";
				hiword.appendChild(hiwordtext);
				pn.insertBefore(before,node);
				pn.insertBefore(hiword,node);
				pn.insertBefore(after,node);
				pn.removeChild(node);
			}
		}
	}
	else if(isNodeTextInputElement(node))
	{
		var tempNodeVal = node.value.toLowerCase();
		if (tempNodeVal == null || tempNodeVal.length == 0){
			return;
		}
		if (tempNodeVal.indexOf(tempWordVal) != -1)
		{
			if(node.className != "searchword")
			{
				node["beforeSearchClassName"] = node.className;
				node.className="searchword";
			}
		}
	}
	else if(node.tagName.toLowerCase() == "select")
	{
		var selectedIndex = node.selectedIndex;
		var options = node.options;
		var option = options[selectedIndex];
		
		if(option != null && option.text != null)
		{
			var tempNodeVal = option.text.toLowerCase();
			if (tempNodeVal == null || tempNodeVal.length == 0){
				return;
			}
			
			if (tempNodeVal.indexOf(tempWordVal) != -1)
			{
				if(node.className != "searchword")
				{
					node["beforeSearchClassName"] = node.className;
					node.className="searchword";
				}
			}
		}
	}
}

function isNodeTextInputElement(node)
{
	if(node.tagName.toLowerCase() == "input" && node.type.toLowerCase()=="text")
	{
		return true;
	}
	return false;	
}

function unHighlightWords(node)
{
	var isIE = (navigator.appName.search(/.*Microsoft\s+Internet\s+Explorer.*/) >= 0); // test for IE
	var spanElements = (node ? node.getElementsByTagName("SPAN") : document.getElementsByTagName("SPAN"));

	for(var i = spanElements.length-1; i >= 0; i--)
	{
		var nextSpanElement = spanElements[i];		

		if (nextSpanElement.className == "searchword")
		{		
			
			var parentNode = nextSpanElement.parentNode;
			parentNode.replaceChild(nextSpanElement.firstChild, nextSpanElement);


			if(isIE) // normalize() method doesn't work on IE5 and crashes IE6 so we have normalize manually
			{
				var curChild = parentNode.lastChild;
				while(curChild)
				{		
					var prevSibling = curChild.previousSibling;		
					if(curChild.nodeType == 3) 
					{											
						if(prevSibling && prevSibling.nodeType == 3)
						{
							prevSibling.nodeValue += curChild.nodeValue;
							parentNode.removeChild(curChild);
						}										
						else if(curChild.nodeValue.length == 0)
						{
							parentNode.removeChild(curChild);
						}
					}								
					curChild = prevSibling;
				}

			}
			else // Mozilla/Firefox normalize() works fine (of course)
			{
				parentNode.normalize();		
			}

		}
	}
	
	var inputElements = (node ? node.getElementsByTagName("INPUT") : document.getElementsByTagName("INPUT"));
	
	for(var i = inputElements.length-1; i >= 0; i--)
	{
		if(inputElements[i].type.toLowerCase()=="text")
		{
			if(inputElements[i].className == "searchword")
			{
				inputElements[i].className = inputElements[i]["beforeSearchClassName"];
			}
		}
	}
	
	var selectElements = (node ? node.getElementsByTagName("SELECT") : document.getElementsByTagName("SELECT"));
	
	for(var i = selectElements.length-1; i >= 0; i--)
	{
		if(selectElements[i].className == "searchword")
		{
				selectElements[i].className = selectElements[i]["beforeSearchClassName"];
		}
	}
}



function tableSearchOnBlur() {
	unHighlightWords();
	return;
}

function tableSearchOnFocus() {
	var targetIds = "form1:jobsTableLeft,form1:jobsTableRight";			
	tableSearchHighlight(targetIds, 'quickFilterInputText');
	return
}

function tableSearchOnKeyup() {
	var targetIds = "form1:jobsTableLeft,form1:jobsTableRight";			
	tableSearchHighlight(targetIds, 'quickFilterInputText');
	return
}


function tableSearchHighlight(targetIds, idSuffix) {

	var ids = targetIds.split(',');
	
	//println("ids: "+ids+" idSuffix: "+idSuffix);

	if (!document.createElement) {
		alert('!createElement');
		return;
	}
			
	var filterInputElement = findElementByTagNameAndIdSuffix("INPUT", idSuffix);
	var searchWord = filterInputElement.value;
	
	//println("searchWord: "+searchWord);
	
	for(i=0; i < ids.length; i++) {
		//println("id["+i+"]: "+ids[i]);
		var tableContents = document.getElementById(ids[i]);
		
		if (tableContents) {	
			unHighlightWords(tableContents);
			if(searchWord.length == 1) continue;	
			if (searchWord != null && searchWord.length > 0) { 
				highlightWord(tableContents, searchWord);
			}
		}
	}
	return;
}

function findElementByTagNameAndIdSuffix(tagName, idSuffix)
{
	var elementList = document.getElementsByTagName(tagName);
	for (var i=0; i < elementList.length; i++) {
	
		var nextElement = elementList[i];
						
		if(nextElement.id.match(new RegExp("^\\S*" + idSuffix + "$")))
			return nextElement;
	}	
}

function getWrappedTableElementContent(toolBarInputElement)
{
	// Walk up to toolBarTable <TABLE> element which contains :quickFilterInputText <INPUT> element
	var toolBarTableElement = toolBarInputElement.parentNode;
	while(toolBarTableElement != null && !toolBarTableElement.id.match(/^(\S*):toolBarTable$/))
		toolBarTableElement = toolBarTableElement.parentNode;
			
	if(toolBarTableElement == null)
		return null;

	// Find next <TABLE> that is a sibling of toolBarTable <TABLE> element.  This will be the wrapped dataTable <TABLE> element
	var wrappedTableElement = toolBarTableElement.nextSibling;
	while(wrappedTableElement != null && wrappedTableElement.nodeName.toUpperCase() != "TABLE")
		wrappedTableElement = wrappedTableElement.nextSibling;
		
	if(wrappedTableElement == null)
		return null;
		
	return wrappedTableElement.tBodies[wrappedTableElement.tBodies.length-1];
}

function selectAllDisplayedRows(objGroup) {
	if(!objGroup) {
		return;
	}
	objGroup.selectAll();
	return;
}

function selectNoneDisplayedRows(objGroup) {
	if(!objGroup) {
		return;
	}
	objGroup.selectNone();
	return;
}

function setSearchHiddenAction(hiddenElementId, action, inputElementId) {
	var hiddenElement = document.getElementById(hiddenElementId);

	if (hiddenElement) {
		println("\nsetSearchHiddenActionNew: "+action);
		hiddenElement.value = action + getHiddenValue(inputElementId);
	}
	return true;
}

function getHiddenValue(hiddenElementId) {
  println("\ngetHiddenValue");
  if (null != hiddenElementId) {
    println("\nhiddenElementId: "+hiddenElementId);
    var hiddenElement = document.getElementById(hiddenElementId);
    println("\nhiddenElement: "+hiddenElement);
    if (hiddenElement) {
      println("\nelementValue: "+hiddenElement.value);
      return hiddenElement.value;
    }
  }
  return "";
}

function doSearchOnComplete(hiddenElementId, buttonId, targetId, event) {
	println("\ndoSearchOnComplete");
	setSearchHiddenAction(hiddenElementId, '');
	println("\ndoSearchOnComplete processing behavior start...");
	hX_6.processBehavior(buttonId, event, 'GET;STOP', namespace + "form1:jobsTable");
	println("\ndoSearchOnComplete processing behavior end...");	
}

function println(s) {
	var console = document.getElementById("console")
	if (console != null) {
		var div = document.createElement("div");
		div.appendChild( document.createTextNode(s));
		console.appendChild(div);
	}
}